/** ioctl.c
 *
 *  Author: J. Krauth
 *  Creation: 23.09.2015
 *  $LastChangedDate: 2015-01-12 17:26:28 +0100 (Mo, 12. Jan 2015) $
 *  $LastChangedRevision: 1381 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#include "board_data.h"
#include "private.h"
#include "pcie040-kapi.h"
#include "pcie040-ioctl.h"

#include <linux/fs.h>
#include <asm/uaccess.h>

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#ifdef CONFIG_XENOMAI

int pcie040_open_rt(struct rtdm_fd *fd, int oflags)
{
	return 0;
}

//--------------------------------------------------------------------------------------

void pcie040_close_rt (struct rtdm_fd *fd)
{
}

#endif

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int pcie040_open(struct inode *inode, struct file *filp)
{
    filp->private_data = p_PCIE040_GetPciDevById(MINOR(inode->i_rdev));
    return 0;
}

int pcie040_release(struct inode *i, struct file *f)
{
    return 0;
}

int pcie040_fasync(int fd, struct file *f, int mode)
{
	return fasync_helper(fd, f, mode, &(PCIE040_PRIVDATA(f->private_data)->asyncQueue));
}

//--------------------------------------------------------------------------------------

#ifdef CONFIG_XENOMAI
	int pcie040_ioctl_rt (struct rtdm_fd *fd, unsigned int request, void *arg)
#else
	#ifdef HAVE_UNLOCKED_IOCTL
		long pcie040_unlocked_ioctl(struct file *filp, unsigned int request, unsigned long arg)
	#else
		int pcie040_ioctl(struct inode *inode, struct file *filp, unsigned int request, unsigned long arg)
	#endif
#endif
{
    int ret = 0;
#ifndef CONFIG_XENOMAI
	struct pci_dev *pdev = (struct pci_dev *)filp->private_data;
#else
	pcie040BoardData_t *board_data = rtdm_fd_device(fd)->device_data;
	struct pci_dev *pdev = board_data->pdev;
#endif

    switch (_IOC_NR(request))
    {
        case _IOC_NR(PCIE040_SET_BOARD_INT_ROUTINE):
        {
            return i_PCIE040_SetBoardIntRoutine(pdev, NULL);
        }

        case _IOC_NR(PCIE040_RESET_BOARD_INT_ROUTINE):
        {
            return i_PCIE040_ResetBoardIntRoutine(pdev);
        }

        case _IOC_NR(PCIE040_TEMPERATURE_TEST_CONVERSION):
		{
        	PCIE040_TemperatureTestConversion_t data;

            ret = i_PCIE040_TemperatureTestConversion(pdev, &data.eoc);
#ifdef CONFIG_XENOMAI
			if (rtdm_fd_is_user(fd))
			{
				if (rtdm_safe_copy_to_user(fd, arg, &data, sizeof(data)))
	                return -EFAULT;
			}
			else
				memcpy(arg, &data, sizeof(data));
#else
            if (copy_to_user ((void __user *)arg, &data, sizeof(data)))
                return -EFAULT;
#endif

            return ret;
		}

        case _IOC_NR(PCIE040_TEMPERATURE_READ_VALUE):
		{
        	PCIE040_TemperatureReadValue_t data;

            ret = i_PCIE040_TemperatureReadValue(pdev, &data.value);

#ifdef CONFIG_XENOMAI
		if (rtdm_fd_is_user(fd))
		{
			if (rtdm_safe_copy_to_user(fd, arg, &data,sizeof(data)))
                return -EFAULT;
		}
		else
			memcpy(arg, &data, sizeof(data));
#else
            if (copy_to_user ((void __user *)arg, &data, sizeof(data)))
                return -EFAULT;
#endif

            return ret;
		}

        case _IOC_NR(PCIE040_TEMPERATURE_WARNING_INIT):
		{
        	PCIE040_TemperatureWarningInit_t data;
#ifdef CONFIG_XENOMAI
            if (rtdm_fd_is_user(fd))
            {
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
            }
            else
            	memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TemperatureWarningInit(pdev, data.value);

            return ret;
		}

        case _IOC_NR(PCIE040_TEMPERATURE_WARNING_START_STOP):
		{
        	PCIE040_TemperatureWarningStartStop_t data;

#ifdef CONFIG_XENOMAI
            if (rtdm_fd_is_user(fd))
            {
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
            }
            else
            	memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TemperatureWarningStartStop(pdev, data.cmd);

            return ret;
		}
		
        case _IOC_NR(PCIE040_TIMER_WATCHDOG_INIT_AND_START):
		{
        	PCIE040_TimerWatchdogInit_t data;

#ifdef CONFIG_XENOMAI
			if (rtdm_fd_is_user(fd))
			{
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
			}
			else
				memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TimerWatchdogInitAndStart(pdev,
													  data.index,
													  data.mode,
													  data.timeUnit,
													  data.timeValue,
													  data.useGate,
													  data.gateLevel,
													  data.useHardwareTrigger,
													  data.hardwareTriggerLevel,
													  data.warningRelayMode,
													  data.warningRelayDelayTimeUnit,
													  data.warningRelayDelayTimeValue,
													  data.useResetRelay,
													  data.resetRelayMode,
													  data.useInterrupt);

            return ret;
		}

        case _IOC_NR(PCIE040_TIMER_WATCHDOG_INIT):
		{
        	PCIE040_TimerWatchdogInit_t data;

#ifdef CONFIG_XENOMAI
			if (rtdm_fd_is_user(fd))
			{
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
			}
			else
				memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TimerWatchdogInit(pdev,
											   data.index,
											   data.mode,
											   data.timeUnit,
											   data.timeValue,
											   data.useGate,
											   data.gateLevel,
											   data.useHardwareTrigger,
											   data.hardwareTriggerLevel,
											   data.warningRelayMode,
											   data.warningRelayDelayTimeUnit,
											   data.warningRelayDelayTimeValue,
											   data.useResetRelay,
											   data.resetRelayMode,
											   data.useInterrupt);

            return ret;
		}

        case _IOC_NR(PCIE040_TIMER_WATCHDOG_START_STOP):
		{
        	PCIE040_TimerWatchdogStartStop_t data;

#ifdef CONFIG_XENOMAI
			if (rtdm_fd_is_user(fd))
			{
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
			}
			else
				memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TimerWatchdogStartStop(pdev,
														   data.index,
														   data.cmd);

            return ret;
		}

        case _IOC_NR(PCIE040_TIMER_WATCHDOG_READ_STATUS_AND_VALUE):
		{
        	PCIE040_TimerWatchdogReadStatusAndValue_t data;

#ifdef CONFIG_XENOMAI
			if (rtdm_fd_is_user(fd))
			{
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
			}
			else
				memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TimerWatchdogReadStatusAndValue(pdev,
														   data.index,
														   &data.value,
														   &data.status,
														   &data.softwareTriggerStatus,
														   &data.hardwareTriggerStatus);

#ifdef CONFIG_XENOMAI
		if (rtdm_fd_is_user(fd))
		{
			if (rtdm_safe_copy_to_user(fd, arg, &data,sizeof(data)))
                return -EFAULT;
		}
		else
			memcpy(arg, &data, sizeof(data));
#else
            if (copy_to_user ((void __user *)arg, &data, sizeof(data)))
                return -EFAULT;
#endif

            return ret;
		}

        case _IOC_NR(PCIE040_TIMER_WATCHDOG_TRIGGER):
		{
        	PCIE040_TimerWatchdogTrigger_t data;

#ifdef CONFIG_XENOMAI
			if (rtdm_fd_is_user(fd))
			{
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
			}
			else
				memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TimerWatchdogTrigger(pdev, data.index);

            return ret;
		}

        case _IOC_NR(PCIE040_TIMER_WATCHDOG_ALL_START_STOP):
		{
        	PCIE040_TimerWatchdogAllStartStop_t data;

#ifdef CONFIG_XENOMAI
			if (rtdm_fd_is_user(fd))
			{
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
			}
			else
				memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TimerWatchdogAllStartStop(pdev, data.mode, data.cmd);

            return ret;
		}

        case _IOC_NR(PCIE040_TIMER_WATCHDOG_ALL_TRIGGER):
		{
        	PCIE040_TimerWatchdogAllTrigger_t data;

#ifdef CONFIG_XENOMAI
            if (rtdm_fd_is_user(fd))
            {
				if (rtdm_safe_copy_from_user(fd, &data, arg, sizeof(data)))
					return -EFAULT;
            }
            else
            	memcpy(&data, arg, sizeof(data));
#else
            if (copy_from_user(&data, (void __user *)arg, sizeof(data)))
            	return -EFAULT;
#endif

            ret = i_PCIE040_TimerWatchdogAllTrigger(pdev, data.mode);

            return ret;
		}

    }

    /* Default case */
    print(KERN_WARNING MODULE_NAME ": Unhandled IOCTL call: %d\n", _IOC_NR(request));
    return -ENOTTY;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

struct file_operations pcie040_fops =
{
    .owner          = THIS_MODULE,
#ifndef CONFIG_XENOMAI
	#ifdef HAVE_UNLOCKED_IOCTL
		.unlocked_ioctl = &pcie040_unlocked_ioctl,
	#else
		.ioctl          = &pcie040_ioctl,
	#endif
		.open           = &pcie040_open,
		.release        = &pcie040_release,
		.fasync         = &pcie040_fasync
#endif
};

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
