/** apci040-ioctl.h
 *
 *  Author: J. Krauth
 *  Creation: 23.09.2015
 *  $LastChangedDate: 2015-02-04 16:34:40 +0100 (Mi, 04. Feb 2015) $
 *  $LastChangedRevision: 1394 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#ifndef __PCIE040_IOCTL_H__
#define __PCIE040_IOCTL_H__

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#define PCIE040_MAGIC 'A'

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#define PCIE040_SET_BOARD_INT_ROUTINE                                     _IO(PCIE040_MAGIC,  1)

#define PCIE040_RESET_BOARD_INT_ROUTINE                                   _IO(PCIE040_MAGIC,  2)

typedef struct __attribute__((__packed__))
{
       uint32_t interruptSource;
       uint32_t interruptMask;
       uint32_t latchedValue;
} PCIE040_TestInterrupt_t;
#define PCIE040_TEST_INTERRUPT                                            _IOR(PCIE040_MAGIC, 3, PCIE040_TestInterrupt_t *)

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t eoc;
} PCIE040_TemperatureTestConversion_t;
#define PCIE040_TEMPERATURE_TEST_CONVERSION                               _IOR(PCIE040_MAGIC, 4, PCIE040_TemperatureTestConversion_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       int32_t value;
} PCIE040_TemperatureReadValue_t;
#define PCIE040_TEMPERATURE_READ_VALUE                                    _IOR(PCIE040_MAGIC, 5, PCIE040_TemperatureReadValue_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       int32_t value;
} PCIE040_TemperatureWarningInit_t;
#define PCIE040_TEMPERATURE_WARNING_INIT                                  _IOW(PCIE040_MAGIC, 6, PCIE040_TemperatureWarningInit_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t cmd;
} PCIE040_TemperatureWarningStartStop_t;
#define PCIE040_TEMPERATURE_WARNING_START_STOP                            _IOW(PCIE040_MAGIC, 7, PCIE040_TemperatureWarningStartStop_t *)

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t index;
       uint8_t mode;
       uint8_t timeUnit;
       uint8_t timeValue;
       uint8_t useGate;
       uint8_t gateLevel;
       uint8_t useHardwareTrigger;
       uint8_t hardwareTriggerLevel;
       uint8_t warningRelayMode;
       uint8_t warningRelayDelayTimeUnit;
       uint8_t warningRelayDelayTimeValue;
       uint8_t useResetRelay;
       uint8_t resetRelayMode;
       uint8_t useInterrupt;
} PCIE040_TimerWatchdogInit_t;
#define PCIE040_TIMER_WATCHDOG_INIT                                       _IOW(PCIE040_MAGIC, 8, PCIE040_TimerWatchdogInit_t *)

//--------------------------------------------------------------------------------------

#define PCIE040_TIMER_WATCHDOG_INIT_AND_START                             _IOW(PCIE040_MAGIC, 14, PCIE040_TimerWatchdogInit_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t index;
       uint8_t cmd;
} PCIE040_TimerWatchdogStartStop_t;
#define PCIE040_TIMER_WATCHDOG_START_STOP                                 _IOW(PCIE040_MAGIC, 9, PCIE040_TimerWatchdogStartStop_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t index;
       uint8_t value;
       uint8_t status;
       uint8_t softwareTriggerStatus;
       uint8_t hardwareTriggerStatus;
} PCIE040_TimerWatchdogReadStatusAndValue_t;
#define PCIE040_TIMER_WATCHDOG_READ_STATUS_AND_VALUE                      _IOWR(PCIE040_MAGIC, 10, PCIE040_TimerWatchdogReadStatusAndValue_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t index;
} PCIE040_TimerWatchdogTrigger_t;
#define PCIE040_TIMER_WATCHDOG_TRIGGER                                    _IOW(PCIE040_MAGIC, 11, PCIE040_TimerWatchdogTrigger_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t mode;
       uint8_t cmd;
} PCIE040_TimerWatchdogAllStartStop_t;
#define PCIE040_TIMER_WATCHDOG_ALL_START_STOP                             _IOW(PCIE040_MAGIC, 12, PCIE040_TimerWatchdogAllStartStop_t *)

//--------------------------------------------------------------------------------------

typedef struct __attribute__((__packed__))
{
       uint8_t mode;
} PCIE040_TimerWatchdogAllTrigger_t;
#define PCIE040_TIMER_WATCHDOG_ALL_TRIGGER                                _IOW(PCIE040_MAGIC, 13, PCIE040_TimerWatchdogAllTrigger_t *)

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#ifdef CONFIG_XENOMAI
/* TODO: ADD (see below)
 *
 * The following IOCTLs are common to all device profiles.
 *
 * Retrieve information about a device or socket.
 * #define 	RTIOC_DEVICE_INFO   _IOR(RTIOC_TYPE_COMMON, 0x00, struct rtdm_device_info)
 *
 * Purge internal device or socket buffers.
 * #define 	RTIOC_PURGE   _IOW(RTIOC_TYPE_COMMON, 0x10, int)
 */
#endif

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#endif // __PCIE040_IOCTL_H__
