
#include "board_data.h"
#include "private.h"

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#ifndef DISABLE_TEMPERATURE_TESTS
    #define ASSERT_TEMPERATURE_PDEV(returnValue, pdev) \
        if (unlikely(pdev == NULL)) \
            return returnValue;

    #define ASSERT_TEMPERATURE_PARAM(returnValue, param, min, max) \
        if (unlikely((param < min) || (param > max))) \
            return returnValue;
#else
    #define ASSERT_TEMPERATURE_PDEV(returnValue, pdev)
    #define ASSERT_TEMPERATURE_PARAM(returnValue, param, min, max)
#endif

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TemperatureTestConversion);

int i_PCIE040_TemperatureTestConversion (struct pci_dev *pdev, uint8_t *eoc)
{
	ASSERT_TEMPERATURE_PDEV(1, pdev)

	*eoc = (uint8_t)(inl (pdev->resource[3].start + 20) & 1);

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TemperatureReadValue);

int i_PCIE040_TemperatureReadValue (struct pci_dev *pdev, int32_t *value)
{
	uint32_t reg = 0;

	ASSERT_TEMPERATURE_PDEV(1, pdev)

	reg = inl (pdev->resource[3].start + 28);

	*value = reg & 0x7F;

	if ((reg & 0x80) != 0)
		*value *= -1;

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TemperatureWarningInit);

int i_PCIE040_TemperatureWarningInit (struct pci_dev *pdev, int32_t value)
{
	ASSERT_TEMPERATURE_PDEV(1, pdev)
	ASSERT_TEMPERATURE_PARAM(2, value, -127, 127)

	if (value < 0)
	{
		value *= -1;
		value |= 0x80;
	}

	/* Disable the temperature acquisition */
	outl (0, pdev->resource[3].start + 8);

	/* Set the warning value */
	outl (0x300, pdev->resource[3].start + 4);
	outl ((value << 8), pdev->resource[3].start);
	outl (0, pdev->resource[3].start + 4);

	/* Start the temperature conversion */
	outl (0x80000, pdev->resource[3].start + 8);

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TemperatureWarningStartStop);

int i_PCIE040_TemperatureWarningStartStop (struct pci_dev *pdev, uint8_t cmd)
{
	ASSERT_TEMPERATURE_PDEV(1, pdev)
	ASSERT_TEMPERATURE_PARAM(2, cmd, 0, 1)

	/* Disable the temperature acquisition */
	outl (0, pdev->resource[3].start + 8);

	/* Enable / Disable warning */
	outl ((cmd << 21), pdev->resource[3].start + 12);

	/* Start the temperature conversion */
	outl (0x80000, pdev->resource[3].start + 8);

	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int i_PCIE040_TemperatureInterruptHandler (struct pci_dev *pdev)
{
	uint32_t reg = 0;
	uint32_t arg[2] = {0};
	int ret = 0;

	reg = inl (pdev->resource[3].start + 16);

	if ((reg & 8) == 8)
	{
		/* Read the channel */
		arg[1] = inl (pdev->resource[3].start + 60);

		/* Read the value */
		arg[0] = inl (pdev->resource[3].start + 60);

		v_PCIE040_DispatchInterrupt (pdev, 4, 0, arg, 2);

		/* The board does interrupt */
		ret = 1;
	}

	return ret;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
