
#include "board_data.h"
#include "private.h"

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#ifndef DISABLE_TIMER_WATCHDOG_TESTS
    #define ASSERT_TIMER_WATCHDOG_PDEV(returnValue, pdev) \
        if (unlikely(pdev == NULL)) \
            return returnValue;

    #define ASSERT_TIMER_WATCHDOG_PARAM(returnValue, param, min, max) \
        if (unlikely((param < min) || (param > max))) \
            return returnValue;

    #define ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(returnValue, check, param, min, max) \
    	if (check) \
		{ \
			if (unlikely((param < min) || (param > max))) \
				return returnValue; \
		}

#else
    #define ASSERT_TIMER_WATCHDOG_PDEV(returnValue, pdev)
    #define ASSERT_TIMER_WATCHDOG_PARAM(returnValue, param, min, max)
	#define ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(returnValue, check, param, min, max)
#endif

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TimerWatchdogInitAndStart);

int i_PCIE040_TimerWatchdogInitAndStart (struct pci_dev *pdev,
                                  uint8_t index,
                                  uint8_t mode,
                                  uint8_t timeUnit,
                                  uint8_t timeValue,
                                  uint8_t useGate,
                                  uint8_t gateLevel,
                                  uint8_t useHardwareTrigger,
                                  uint8_t hardwareTriggerLevel,
                                  uint8_t warningRelayMode,
                                  uint8_t warningRelayDelayTimeUnit,
                                  uint8_t warningRelayDelayTimeValue,
                                  uint8_t useResetRelay,
                                  uint8_t resetRelayMode,
                                  uint8_t useInterrupt)
{
	uint32_t cmd = 0;

	ASSERT_TIMER_WATCHDOG_PDEV(1, pdev)
	ASSERT_TIMER_WATCHDOG_PARAM(2, index, 0, 6)
	ASSERT_TIMER_WATCHDOG_PARAM(3, mode, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM(4, timeUnit, 1, 4)
	ASSERT_TIMER_WATCHDOG_PARAM(5, timeValue, 1, 255)
	ASSERT_TIMER_WATCHDOG_PARAM(6, useGate, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(7, useGate, gateLevel, 1, 2)
	ASSERT_TIMER_WATCHDOG_PARAM(8, useHardwareTrigger, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(9, useHardwareTrigger, hardwareTriggerLevel, 1, 3)
	ASSERT_TIMER_WATCHDOG_PARAM(10, warningRelayMode, 0, 2)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(11, (warningRelayMode == 1), warningRelayDelayTimeUnit, 1, 3)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(12, (warningRelayMode == 1), warningRelayDelayTimeValue, 1, 255)
	ASSERT_TIMER_WATCHDOG_PARAM(13, useResetRelay, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(14, useResetRelay, resetRelayMode, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM(15, useInterrupt, 0, 1)

	/* Stop the timer / watchdog */
	outl (0, pdev->resource[2].start + 16 + (index * 32) + 12);

	/* Write the time unit */
	outl (timeUnit, pdev->resource[2].start + 16 + (index * 32) + 8);

	/* Write the reload value */
	outl (timeValue, pdev->resource[2].start + 16 + (index * 32) + 4);

	/* Timer mode */
	if (mode == 1)
		cmd = (uint32_t)((2 << 13) | (1 << 4));
	else
	{
		/* Watchdog mode 2, the warning relais has to be set when
		 * the watchdog starts and reseted when the watchdog runs down
		 */
		if (warningRelayMode == 2)
			cmd = (uint32_t)(3 << 13);
	}

	if (useGate == 1)
		cmd |= (gateLevel << 7);

	if (useHardwareTrigger)
		cmd |= (hardwareTriggerLevel << 5);

	if ((warningRelayMode == 1) && (index == 0))
	{
		outl (warningRelayDelayTimeUnit, pdev->resource[2].start + 16 + 28);
		outl (warningRelayDelayTimeValue, pdev->resource[2].start + 16 + 24);
	}
	
	if (warningRelayMode > 0)
		cmd |= (1 << 2);	

	if (resetRelayMode)
		cmd |= (resetRelayMode << 23) | (1 << 3);

	if (useInterrupt)
		cmd |= 2;
	
	/* Enable and trigger the watchdog */
	cmd |= 0x201;

	outl (cmd, pdev->resource[2].start + 16 + (index * 32) + 12);

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TimerWatchdogInit);

int i_PCIE040_TimerWatchdogInit (struct pci_dev *pdev,
                                  uint8_t index,
                                  uint8_t mode,
                                  uint8_t timeUnit,
                                  uint8_t timeValue,
                                  uint8_t useGate,
                                  uint8_t gateLevel,
                                  uint8_t useHardwareTrigger,
                                  uint8_t hardwareTriggerLevel,
                                  uint8_t warningRelayMode,
                                  uint8_t warningRelayDelayTimeUnit,
                                  uint8_t warningRelayDelayTimeValue,
                                  uint8_t useResetRelay,
                                  uint8_t resetRelayMode,
                                  uint8_t useInterrupt)
{
	uint32_t cmd = 0;

	ASSERT_TIMER_WATCHDOG_PDEV(1, pdev)
	ASSERT_TIMER_WATCHDOG_PARAM(2, index, 0, 6)
	ASSERT_TIMER_WATCHDOG_PARAM(3, mode, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM(4, timeUnit, 1, 4)
	ASSERT_TIMER_WATCHDOG_PARAM(5, timeValue, 1, 255)
	ASSERT_TIMER_WATCHDOG_PARAM(6, useGate, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(7, useGate, gateLevel, 1, 2)
	ASSERT_TIMER_WATCHDOG_PARAM(8, useHardwareTrigger, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(9, useHardwareTrigger, hardwareTriggerLevel, 1, 3)
	ASSERT_TIMER_WATCHDOG_PARAM(10, warningRelayMode, 0, 2)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(11, (warningRelayMode == 1), warningRelayDelayTimeUnit, 1, 3)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(12, (warningRelayMode == 1), warningRelayDelayTimeValue, 1, 255)
	ASSERT_TIMER_WATCHDOG_PARAM(13, useResetRelay, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM_DEPEND(14, useResetRelay, resetRelayMode, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM(15, useInterrupt, 0, 1)

	/* Stop the timer / watchdog */
	outl (0, pdev->resource[2].start + 16 + (index * 32) + 12);

	/* Write the time unit */
	outl (timeUnit, pdev->resource[2].start + 16 + (index * 32) + 8);

	/* Write the reload value */
	outl (timeValue, pdev->resource[2].start + 16 + (index * 32) + 4);

	/* Timer mode */
	if (mode == 1)
		cmd = (uint32_t)((2 << 13) | (1 << 4));
	else
	{
		/* Watchdog mode 2, the warning relais has to be set when
		 * the watchdog starts and reseted when the watchdog runs down
		 */
		if (warningRelayMode == 2)
			cmd = (uint32_t)(3 << 13);
	}

	if (useGate == 1)
		cmd |= (gateLevel << 7);

	if (useHardwareTrigger)
		cmd |= (hardwareTriggerLevel << 5);

	if ((warningRelayMode == 1) && (index == 0))
	{
		outl (warningRelayDelayTimeUnit, pdev->resource[2].start + 16 + 28);
		outl (warningRelayDelayTimeValue, pdev->resource[2].start + 16 + 24);
	}
	
	if (warningRelayMode > 0)
		cmd |= (1 << 2);	

	if (resetRelayMode)
		cmd |= (resetRelayMode << 23) | (1 << 3);

	if (useInterrupt)
		cmd |= 2;

	outl (cmd, pdev->resource[2].start + 16 + (index * 32) + 12);

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TimerWatchdogStartStop);

int i_PCIE040_TimerWatchdogStartStop (struct pci_dev *pdev, uint8_t index, uint8_t cmd)
{
	uint32_t reg = 0;

	ASSERT_TIMER_WATCHDOG_PDEV(1, pdev)
	ASSERT_TIMER_WATCHDOG_PARAM(2, index, 0, 6)
	ASSERT_TIMER_WATCHDOG_PARAM(3, cmd, 0, 1)

	reg = inl (pdev->resource[2].start + 16 + (index * 32) + 12);

	/* Start / Stop the timer / watchdog */
	outl ((reg & ~0x601 ) | cmd, pdev->resource[2].start + 16 + (index * 32) + 12);

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TimerWatchdogReadStatusAndValue);

int i_PCIE040_TimerWatchdogReadStatusAndValue (struct pci_dev *pdev,
                                                uint8_t index,
                                                uint8_t *value,
                                                uint8_t *status,
                                                uint8_t *softwareTriggerStatus,
                                                uint8_t *hardwareTriggerStatus)
{
	uint32_t reg = 0;

	ASSERT_TIMER_WATCHDOG_PDEV(1, pdev)
	ASSERT_TIMER_WATCHDOG_PARAM(2, index, 0, 6)

	*value = (uint8_t) inl (pdev->resource[2].start + 16 + (index * 32));
	reg = inl (pdev->resource[2].start + 16 + (index * 32) + 16);
	*status = (uint8_t)(reg & 1);
	*softwareTriggerStatus = (uint8_t)((reg >> 1) & 1);
	*hardwareTriggerStatus = (uint8_t)((reg >> 2) & 1);

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TimerWatchdogTrigger);

int i_PCIE040_TimerWatchdogTrigger (struct pci_dev *pdev, uint8_t index)
{
	uint32_t reg = 0;

	ASSERT_TIMER_WATCHDOG_PDEV(1, pdev)
	ASSERT_TIMER_WATCHDOG_PARAM(2, index, 0, 6)

	reg = inl (pdev->resource[2].start + 16 + (index * 32) + 12);
	outl ((reg & ~0x600 ) | 0x200, pdev->resource[2].start + 16 + (index * 32) + 12);

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TimerWatchdogAllStartStop);

int i_PCIE040_TimerWatchdogAllStartStop (struct pci_dev *pdev, uint8_t mode, uint8_t cmd)
{
	ASSERT_TIMER_WATCHDOG_PDEV(1, pdev)
	ASSERT_TIMER_WATCHDOG_PARAM(2, mode, 0, 1)
	ASSERT_TIMER_WATCHDOG_PARAM(3, cmd, 0, 1)

	if (mode == 1)
	{
		if (cmd == 1)
			outl (0x8, pdev->resource[2].start + 16);
		else
			outl (0x10, pdev->resource[2].start + 16);
	}
	else
	{
		if (cmd == 1)
			outl (0x1, pdev->resource[2].start + 16);
		else
			outl (0x2, pdev->resource[2].start + 16);
	}

	return 0;
}

//--------------------------------------------------------------------------------------

EXPORT_SYMBOL(i_PCIE040_TimerWatchdogAllTrigger);

int i_PCIE040_TimerWatchdogAllTrigger (struct pci_dev *pdev, uint8_t mode)
{
	ASSERT_TIMER_WATCHDOG_PDEV(1, pdev)
	ASSERT_TIMER_WATCHDOG_PARAM(2, mode, 0, 1)

	if (mode == 1)
		outl (0x20, pdev->resource[2].start + 16);
	else
		outl (0x4, pdev->resource[2].start + 16);

	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogInterruptHandler (struct pci_dev *pdev)
{
	uint32_t reg = 0;
	uint32_t interruptSource = 0;
	uint32_t interruptMask = 0;
	int ret = 0;
	int i = 0;

	for (i = 0; i < 7; i ++)
	{
		reg = inl (pdev->resource[2].start + 16 + (i * 32) + 20);

		if ((reg & 1) == 1)
		{
			outl (reg, pdev->resource[2].start + 16 + (i * 32) + 20);

			/* Get the mode */
			reg = inl (pdev->resource[2].start + 16 + (i * 32) + 12);

			/* Test if timer */
			if (reg & 0x10)
				interruptSource = 0x1;
			else
				interruptSource = 0x2;

			interruptMask |= (1 << i);

			v_PCIE040_DispatchInterrupt (pdev, interruptSource, interruptMask, NULL, 0);

			/* The board does interrupt */
			ret = 1;
		}
	}

	return ret;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

