#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <sys/mman.h>
#include <sys/syscall.h>

#include <pcie040.h>

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

volatile uint8_t quit = 0;

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

void quit_signal (int sig)
{
	quit = 1;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

void * task_body (void *arg)
{
	uint8_t eoc = 0;
	uint32_t value = 0;
	int ret = 0;
	int last = 200;

	struct timespec tp;
	char buffer [80];
	struct itimerspec new_value;
	struct sigaction action;
	struct sigevent sigev;
	sigset_t set;
	int signum = SIGALRM;
	timer_t timer;

	sigev.sigev_notify = SIGEV_THREAD_ID;
	sigev.sigev_notify_thread_id = syscall(SYS_gettid);
	sigev.sigev_signo = signum;

	sigemptyset(&set);
	sigaddset(&set, signum);
	sigprocmask(SIG_BLOCK, &set, NULL);

	if (timer_create (CLOCK_MONOTONIC, &sigev, &timer) == -1)
	{
		perror ("timer_create");
		goto end;
	}

	new_value.it_interval.tv_sec = 0;
	new_value.it_interval.tv_nsec = 500000000llu;	// 500 ms
	new_value.it_value.tv_sec = 0;
	new_value.it_value.tv_nsec = 500000000llu;		// 500 ms

	if (timer_settime (timer, 0, &new_value, NULL) == -1)
	{
		perror ("timer_settime");
		goto end;
	}

	while (!quit)
	{
		/* wait for signal */
		if (sigwait (&set, &signum) == -1)
		{
			perror ("sigwait");
			goto end;
		}

		if ((ret = i_PCIE040_TemperatureTestConversion (0, &eoc)) != 0)
		{
			if (ret < 0)
				perror ("i_PCIE040_TemperatureTestConversion");
			else
				printf("i_PCIE040_TemperatureTestConversion error %i\n", ret);

			quit = 1;
		}

		if (eoc)
		{
			if ((ret = i_PCIE040_TemperatureReadValue (0, &value)) != 0)
			{
				if (ret < 0)
					perror ("i_PCIE040_TemperatureReadValue");
				else
					printf("i_PCIE040_TemperatureReadValue error %i\n", ret);

				quit = 1;
			}

			/* Print only if the temperature changed */
			if (value != last)
				printf ("Temperature %i (degree Celsius)\n", value);

			last = value;
		}
    }

	end:
		quit = 1;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int main (int argc, char *argv[])
{
	int ret = 0;
	pthread_t thread;
	uint8_t boardCount = 0;

	if ((ret = i_PCIE040_GetNumberOfBoards (&boardCount)) != 0)
	{
		printf("i_PCIE040_GetNumberOfBoards error %i\n", ret);
		exit(1);
	}

	if (boardCount == 0)
	{
		printf ("No board found\n");
		exit(1);
	}

	printf ("Number of boards: %i\n", boardCount);

	/* Create the timer thread  */
	if ((ret = pthread_create (&thread, NULL, task_body, NULL)) != 0)
	{
		perror("pthread_create\n");
		goto end;
	}

	/* Manage Ctrl + C */
	signal (SIGINT, quit_signal);

	/* Wait that the customer use the combination Ctrl + C to quit the sample */
	while (!quit)
		sleep (1);

end:
	quit = 1;

	pthread_join(thread, NULL);

	/* Close the device */
	if ((ret = i_PCIE040_CloseBoard (0)) != 0)
		printf("i_PCIE040_CloseBoard error %i\n", ret);

	return 0;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
