/** temperature_read.c
 *
 *  Author: J. Krauth
 *  Creation: 22.09.2015
 *  $LastChangedDate: 2014-12-15 15:55:58 +0100 (Mo, 15. Dez 2014) $
 *  $LastChangedRevision: 1327 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

#include <linux/version.h>
#include <generated/autoconf.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/kthread.h>
#include <linux/wait.h>

#include "../driver/pcie040-kapi.h"

MODULE_LICENSE("GPL");
MODULE_AUTHOR("ADDI-DATA GmbH <info@addi-data.com>");
MODULE_DESCRIPTION("APCIe-040");

static struct task_struct *task = NULL;
static wait_queue_head_t wait_queue;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

static __inline__ uint8_t endOfConversion (struct pci_dev *pdev)
{
	uint8_t eoc = 0;

	/* Check if a temperature value is ready to be read */
	i_PCIE040_TemperatureTestConversion (pdev, &eoc);

	return eoc;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

int thread_function (void *data)
{
	struct pci_dev *pdev = data;
	int32_t value = 0;

	while (!kthread_should_stop())
	{
		/* Wait the end of the conversion */
		if (wait_event_interruptible_timeout (wait_queue, (endOfConversion (pdev) != 0), (HZ/10)) == 0)
		{
			printk ("wait_event_interruptible_timeout: timeout occured\n");
			continue;
		}

		/* A value is ready, read it */
		i_PCIE040_TemperatureReadValue (pdev, &value);
		printk ("Temperature %i (degree Celsius)\n", value);

		schedule ();
	}

	printk ("%s end\n", __FUNCTION__);
	do_exit (0);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

static int __init sample_init(void)
{
	struct pci_dev *pdev = NULL;

	/* Get a pointer on the board index 0 */
	if ((pdev = p_PCIE040_GetPciDevById (0)) == NULL)
	{
		printk ("p_PCIE040_GetPciDevById error\n");
		return -ENXIO;
	}

	/* Set a waitqueue to wait the end of conversion */
	init_waitqueue_head (&wait_queue);

	/* Create a thread */
	if ((task = kthread_create (thread_function, pdev, "thread_function")) == NULL)
	{
		printk ("kthread_create error\n");
		return -ECANCELED;
	}

	/* Start the thread */
	wake_up_process (task);

	return 0;
}

//------------------------------------------------------------------------------

static void __exit sample_exit(void)
{
	if (task)
		kthread_stop (task);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

module_exit(sample_exit);
module_init(sample_init);
