/** timer.c
 *
 *  Author: J. Krauth
 *  Creation: 24.09.2015
 *  $LastChangedDate: 2014-12-15 15:55:58 +0100 (Mo, 15. Dez 2014) $
 *  $LastChangedRevision: 1327 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

#include <linux/version.h>
#include <generated/autoconf.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/kthread.h>
#include <linux/delay.h>

#include "../driver/pcie040-kapi.h"

MODULE_LICENSE("GPL");
MODULE_AUTHOR("ADDI-DATA GmbH <info@addi-data.com>");
MODULE_DESCRIPTION("APCIe-040");

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

static struct task_struct *task = NULL;
static struct pci_dev *pdev = NULL;
static uint8_t index = 0;
static uint8_t useInterrupt = 0;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

void pcie040_isr (struct pci_dev *pdev, uint8_t fifoOverflow, uint32_t interruptSource, uint32_t interruptMask, uint32_t *value, uint32_t count)
{
	if (interruptSource & 0x1)
	{
		if (interruptMask & 1)
		{
			printk ("Timer 0 interrupt\n");
		}
		if (interruptMask & 2)
		{
			printk ("Timer 1 interrupt\n");
		}
		if (interruptMask & 4)
		{
			printk ("Timer 2 interrupt\n");
		}
		if (interruptMask & 8)
		{
			printk ("Timer 3 interrupt\n");
		}
		if (interruptMask & 0x10)
		{
			printk ("Timer 4 interrupt\n");
		}
		if (interruptMask & 0x20)
		{
			printk ("Timer 5 interrupt\n");
		}
		if (interruptMask & 0x40)
		{
			printk ("Timer 6 interrupt\n");
		}
	}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

int thread_function (void *data)
{
	int ret = 0;
	uint8_t value = 0;
	uint8_t status = 0;
	uint8_t softwareTriggerStatus = 0;
	uint8_t hardwareTriggerStatus = 0;
	uint8_t last[4] = {0};

	while (!kthread_should_stop())
	{
		ret = i_PCIE040_TimerWatchdogReadStatusAndValue (pdev,
		                                                  index,
		                                                  &value,
		                                                  &status,
		                                                  &softwareTriggerStatus,
		                                                  &hardwareTriggerStatus);

		if (ret != 0)
		{
			printk ("i_PCIE040_TimerWatchdogReadStatusAndValue error: %d\n", ret);
			do_exit (0);
		}

		if ((value != last[0]) || (status != last[1]) || (softwareTriggerStatus != last[2]) || (hardwareTriggerStatus != last[3]))
		{
			printk ("The timer status is %x, value is %u, soft trigger is %x, hard trigger is %x \n", status, value, softwareTriggerStatus, hardwareTriggerStatus);

			/* Use to update the display only when a value changes */
			last[0] = value;
			last[1] = status;
			last[2] = softwareTriggerStatus;
			last[3] = hardwareTriggerStatus;
		}

		schedule ();
	}

	printk ("%s end\n", __FUNCTION__);
	do_exit (0);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

static int __init sample_init(void)
{
	int ret = 0;
	uint8_t mode = 1;							// 0: Watchdog, 1: timer
	uint8_t timeUnit = 3;						// 1: us, 2: ms, 3: s, 4: min
	uint8_t timeValue = 5;						// 1 to 255
	uint8_t useGate = 0;						// 0: no, 1: yes
	uint8_t gateLevel = 0;						// 1: low active, 2: high active

	index = 0;		  // 0 to 6
	useInterrupt = 1; // 0: no, 1: yes

	/* Get a pointer on the board index 0 */
	if ((pdev = p_PCIE040_GetPciDevById (0)) == NULL)
	{
		printk ("p_PCIE040_GetPciDevById error\n");
		return -ENXIO;
	}

	/* Create a thread */
	if ((task = kthread_create (thread_function, NULL, "thread_function")) == NULL)
	{
		printk ("kthread_create error\n");
		return -ECANCELED;
	}

	if (useInterrupt)
	{
		ret = i_PCIE040_SetBoardIntRoutine (pdev, pcie040_isr);

		if (ret != 0)
		{
			printk ("i_PCIE040_SetBoardIntRoutine error: %d\n", ret);
			return -ECANCELED;
		}
	}

	ret = i_PCIE040_TimerWatchdogInit (pdev,
										index,
										mode,
										timeUnit,
										timeValue,
										useGate,
										gateLevel,
										0,
										0,
										0,
										0,
										0,
										0,
										0,
										useInterrupt);

	if (ret != 0)
	{
		printk ("i_PCIE040_TimerWatchdogInit error: %d\n", ret);
		i_PCIE040_ResetBoardIntRoutine (pdev);
		return -ECANCELED;
	}

	/* Start the timer */
	i_PCIE040_TimerWatchdogStartStop (pdev, index, 1);

	/* Start the thread */
	wake_up_process (task);

	return 0;
}

//------------------------------------------------------------------------------

static void __exit sample_exit(void)
{

	i_PCIE040_TimerWatchdogStartStop (pdev, index, 0);

	if (useInterrupt)
		i_PCIE040_ResetBoardIntRoutine (pdev);

	if (task)
		kthread_stop (task);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

module_exit(sample_exit);
module_init(sample_init);
