/** pcie040.c
 *
 *  Author: J. Krauth
 *  Creation: 23.09.2015
 *  $LastChangedDate: 2015-02-04 16:34:40 +0100 (Mi, 04. Feb 2015) $
 *  $LastChangedRevision: 1394 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#include <dirent.h>
#include <unistd.h>
#include "common.h"

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int _boards[16];

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int i_PCIE040_GetNumberOfBoards(uint8_t *boardCount)
{
    DIR *dp;
    struct dirent *ep;

    *boardCount = 0;
    dp = opendir ("/dev/rtdm/");

    if (dp == NULL)
        return -1;

    while ((ep = readdir(dp)) != NULL)
        if (strstr(ep->d_name, "pcie040_") != NULL)
            ++(*boardCount);

    closedir(dp);
    return 0;
}

//--------------------------------------------------------------------------------------

int i_PCIE040_GetBoardFileDescriptor(uint8_t boardId)
{
    if (boardId > sizeof(_boards) / sizeof(*_boards))
        return -1;
    else if (_boards[boardId] > 0)
        return _boards[boardId];
    else
    {
        const char devPrefix[] = "/dev/rtdm/pcie040_%u";
        char devPath[sizeof(devPrefix)];
        sprintf(devPath, devPrefix, boardId);
        _boards[boardId] = open(devPath, 0);
        return _boards[boardId];
    }
}

//--------------------------------------------------------------------------------------

int i_PCIE040_CloseBoard(uint8_t boardId)
{
    int fd = _boards[boardId];
    _boards[boardId] = -1;
    return close(fd);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
