/** timer_watchdog.c
 *
 *  Author: J. Krauth
 *  Creation: 23.09.2015
 *  $LastChangedDate: 2015-01-13 10:48:53 +0100 (Di, 13. Jan 2015) $
 *  $LastChangedRevision: 1385 $
 *
 *  Copyright (C) 2015 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmünster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 **/

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

#include "common.h"

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogInitAndStart (uint8_t boardId,
                                         uint8_t index,
                                         uint8_t mode,
                                         uint8_t timeUnit,
                                         uint8_t timeValue,
                                         uint8_t useGate,
                                         uint8_t gateLevel,
                                         uint8_t useHardwareTrigger,
                                         uint8_t hardwareTriggerLevel,
                                         uint8_t warningRelayMode,
                                         uint8_t warningRelayDelayTimeUnit,
                                         uint8_t warningRelayDelayTimeValue,
                                         uint8_t useResetRelay,
                                         uint8_t resetRelayMode,
                                         uint8_t useInterrupt)
{
	int ret = 0;

	PCIE040_TimerWatchdogInit_t params =
    {
        .index = index,
        .mode = mode,
        .timeUnit = timeUnit,
        .timeValue = timeValue,
        .useGate = useGate,
        .gateLevel = gateLevel,
        .useHardwareTrigger = useHardwareTrigger,
        .hardwareTriggerLevel = hardwareTriggerLevel,
        .warningRelayMode = warningRelayMode,
        .warningRelayDelayTimeUnit = warningRelayDelayTimeUnit,
        .warningRelayDelayTimeValue = warningRelayDelayTimeValue,
        .useResetRelay = useResetRelay,
        .resetRelayMode = resetRelayMode,
        .useInterrupt = useInterrupt
    };

    ret = ioctl(i_PCIE040_GetBoardFileDescriptor(boardId), PCIE040_TIMER_WATCHDOG_INIT_AND_START, &params);

	return ret;
}

//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogInit (uint8_t boardId,
                                  uint8_t index,
                                  uint8_t mode,
                                  uint8_t timeUnit,
                                  uint8_t timeValue,
                                  uint8_t useGate,
                                  uint8_t gateLevel,
                                  uint8_t useHardwareTrigger,
                                  uint8_t hardwareTriggerLevel,
                                  uint8_t warningRelayMode,
                                  uint8_t warningRelayDelayTimeUnit,
                                  uint8_t warningRelayDelayTimeValue,
                                  uint8_t useResetRelay,
                                  uint8_t resetRelayMode,
                                  uint8_t useInterrupt)
{
	int ret = 0;

	PCIE040_TimerWatchdogInit_t params =
    {
        .index = index,
        .mode = mode,
        .timeUnit = timeUnit,
        .timeValue = timeValue,
        .useGate = useGate,
        .gateLevel = gateLevel,
        .useHardwareTrigger = useHardwareTrigger,
        .hardwareTriggerLevel = hardwareTriggerLevel,
        .warningRelayMode = warningRelayMode,
        .warningRelayDelayTimeUnit = warningRelayDelayTimeUnit,
        .warningRelayDelayTimeValue = warningRelayDelayTimeValue,
        .useResetRelay = useResetRelay,
        .resetRelayMode = resetRelayMode,
        .useInterrupt = useInterrupt
    };

    ret = ioctl(i_PCIE040_GetBoardFileDescriptor(boardId), PCIE040_TIMER_WATCHDOG_INIT, &params);

	return ret;
}

//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogStartStop (uint8_t boardId,
                                       uint8_t index,
                                       uint8_t cmd)
{
	int ret = 0;

	PCIE040_TimerWatchdogStartStop_t params =
    {
        .index = index,
        .cmd = cmd
    };

    ret = ioctl(i_PCIE040_GetBoardFileDescriptor(boardId), PCIE040_TIMER_WATCHDOG_START_STOP, &params);

	return ret;
}

//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogReadStatusAndValue (uint8_t boardId,
                                                uint8_t index,
                                                uint8_t *value,
                                                uint8_t *status,
                                                uint8_t *softwareTriggerStatus,
                                                uint8_t *hardwareTriggerStatus)
{
	PCIE040_TimerWatchdogReadStatusAndValue_t params =
	{
		 .index = index
	};

    int ret = ioctl(i_PCIE040_GetBoardFileDescriptor(boardId), PCIE040_TIMER_WATCHDOG_READ_STATUS_AND_VALUE, &params);

    if (ret == 0)
    {
        *value = params.value;
        *status = params.status;
        *softwareTriggerStatus = params.softwareTriggerStatus;
        *hardwareTriggerStatus = params.hardwareTriggerStatus;
    }
    else
    {
        *value = 0;
        *status = 0;
        *softwareTriggerStatus = 0;
        *hardwareTriggerStatus = 0;
    }

    return ret;
}

//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogTrigger (uint8_t boardId,
                                     uint8_t index)
{
	int ret = 0;

	PCIE040_TimerWatchdogTrigger_t params =
	{
		 .index = index
	};

    ret = ioctl(i_PCIE040_GetBoardFileDescriptor(boardId), PCIE040_TIMER_WATCHDOG_TRIGGER, &params);

	return ret;
}

//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogAllStartStop (uint8_t boardId,
                                          uint8_t mode,
                                          uint8_t cmd)
{
	int ret = 0;

	PCIE040_TimerWatchdogAllStartStop_t params =
	{
		 .mode = mode,
		 .cmd = cmd
	};

    ret = ioctl(i_PCIE040_GetBoardFileDescriptor(boardId), PCIE040_TIMER_WATCHDOG_ALL_START_STOP, &params);

	return ret;
}

//--------------------------------------------------------------------------------------

int i_PCIE040_TimerWatchdogAllTrigger (uint8_t boardId,
                                        uint8_t mode)
{
	int ret = 0;

	PCIE040_TimerWatchdogAllTrigger_t params =
	{
		 .mode = mode
	};

    ret = ioctl(i_PCIE040_GetBoardFileDescriptor(boardId), PCIE040_TIMER_WATCHDOG_ALL_TRIGGER, &params);

	return ret;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

